//=======================================================================
// Basic C++: Index et rfrences croises : la classe Crossref
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: Crossref.h,v 1.1 2005/05/03 12:50:31 jpr Exp $
//=======================================================================

#ifndef _CROSSREF_H_
#define _CROSSREF_H_

#include <iostream> 
#include <string>
#include <map>
#include <set>
using namespace std;

// La classe Crossref est aussi juste une encapsulation de map<>

class Crossref
{
public:

    // Exception (ligne non trouve)

    class Not_Found {};

    // Le type de la map<> interne

    typedef map<int, set<string> > Map;

    // Raccourci pour donner directement accs aux itrateurs de la map
    // On n'a pas besoin de parcourir pour modifier, donc on fournit juste un
    // const_iterator.

    typedef Map::const_iterator const_iterator;

    // Une Crossref se construit  partir d'un Index

    Crossref(const Index& index);

    // Accesseur pour obtenir l'ensemble des mots d'une ligne

    const set<string>& operator[](int ln) const;

    // Pour utiliser directement Crossref dans les algorithmes, comme si c'tait
    // une collection de la STL, on donne accs aux itrateurs et  la taille

    const_iterator begin() const {return _crossref.begin();}
    const_iterator end() const {return _crossref.end();}
    unsigned int size() const {return _crossref.size();}

    // Oprateur d'affichage : on ne peut pas utiliser la fonction display()
    // des collections, car set<> n'a pas d'oprateur <<

    friend ostream& operator<<(ostream& os, const Crossref& cref);

private:

    // La table de cross-references elle-mme

    Map _crossref;

    // Une classe-fonction pour construire commodment une Crossref  l'aide
    // d'un Index

    struct make_crossref
    {
        // Une rfrence sur la map  remplir
        Crossref& _cr;

        // Le constructeur initialise juste la rfrence sur la map  construire
        make_crossref(Crossref& cr) : _cr(cr) {}

        // L'oprateur () fait le travail, lment de l'index par lment. Il
        // roit donc une paire d type de celles de l'Index.
        void operator()(pair<string, set<int> > p);
    };
};

#endif
