//=======================================================================
// Basic C++: Index et rfrences croises
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: main_Index.cpp,v 1.2 2005/05/03 12:50:31 jpr Exp $
//=======================================================================

#include <iostream> 
#include <sstream>
#include <list>
#include <vector>
#include <algorithm>
#include <cctype>
using namespace std;

#include "stl_extensions.h"
using namespace JPR_STL_EXTENSIONS; 

#include "Index.h"
#include "Crossref.h"

//-----------------------------------------------------------------------
// Fonctions et classes-fonctions utilises par main()
//-----------------------------------------------------------------------

// Un objet-function applicable aux lments d'une string, remplaant chaque
// caractre non alphanumrique par un espace.

struct punct_to_space
{
    char operator()(char c) const
    {
        return isalpha(c) ? c : ' ';
    }
};

// Un objet-fonction pour transformer une entre de l'index en une paire
// (mot, nb de lignes o le mot apparait)

struct nlines
{    
    pair<string, int> operator()(pair<string, set<int> > p) const
    {
        return make_pair(p.first, p.second.size());
    }
};

// Un oprateur ad hoc de comparaison des paires
// ATTENTION : il *doit* tre dans le namepsace std pour que les fonctionnelles
// le trouvent ("Koenig's lookup")

namespace std
{
    bool operator<(pair<string, int> p1, pair<string, int> p2)
    {
        return p1.second < p2.second;
    }
}

//-----------------------------------------------------------------------------
// Constantes globales
//-----------------------------------------------------------------------------

const unsigned int LWORD = 2;  		// Longueur minimale d'un mot e l'index
const unsigned int MAXLINE = 1024; 	// Longueur maximale d'une ligne 

//---------------------------------------------- -------------------------------
// Programme principal  
//-----------------------------------------------------------------------------

int main() 
{
    string word;
    Index the_index; 
    char buffer[MAXLINE + 1];

    // On lit l'entre standard ligne par ligne, on remplace les caractres non
    // alphanumriques par des espaces et on relit la chaine obtenue mot par
    // mot. Puis on met les mot en question dans l'index

    for (int ln = 1; (cin.getline(buffer, MAXLINE)); ++ln)
    {
        // On transforme la ligne en string, par commodit
        string line(buffer);
        
        // limination des caractres non alphanumriques : on transforme la
        // chaine sur place
	transform(line.begin(), line.end(), line.begin(), punct_to_space());

        // On transforme la ligne en istringstream et on la lit par string en
        // liminant les mots trop cours (cad de longueur < LWORD)
        istringstream is(line);
        string word;
        while (is >> word)
        {
            if (word.size() >= LWORD) the_index.add(word, ln);
        }
    }

    // Affichage de l'index obtenu

    cout << "Index\n"
         << "-----\n"
         << the_index << endl;

    // Constitution et affichage des rfrences croises

    Crossref the_crossref(the_index);
    cout << "Rfrences croises\n"
         << "-------------------\n"
         << the_crossref << endl;

    // Mots dans la ligne 4

    cout << "Mots de la ligne 4\n"
         << "------------------\n";
    display(cout, the_crossref[4]);
    cout << endl;

    // Recherche du nombre de lignes o apparait un mot
    // On utilise un vector ici, car plus facile  trier sur place.

    vector<pair<string, int> > the_nlines;
    transform(the_index.begin(), the_index.end(), back_inserter(the_nlines), nlines());
    cout << "Nombre de lignes pour chaque mot : tri croissant\n"
         << "------------------------------------------------\n";   
    sort(the_nlines.begin(), the_nlines.end(), less<pair<string, int> >());
    display(cout, the_nlines, "\n");

    // Mots apparaissant dans strictement plus d'une ligne

    list<pair<string, int> > the_frequents;
    copy_if(the_nlines.begin(), the_nlines.end(), back_inserter(the_frequents),
            bind2nd(greater<pair<string, int> >(), make_pair("", 1)));
    cout << "Mots apparaissant dans strictement plus d'une ligne\n"
         << "---------------------------------------------------\n";   
    display(cout, the_frequents, "\n");


    return 0;
}

