/*
//====================================================================
// 	     Calculette avec symboles: analyse syntaxique
//====================================================================
//      	 Jean-Paul RIGAULT --- Jeanvier 1993
//====================================================================
*/

%{
#ifdef __GNUC__
#include <alloca.h>
#endif /* __GNUC__ */

#include <iostream>
    using namespace std;

#include "Expr.h"
#include "Ternary.h"
#include "Modulo.h"
#include "Variable.h"

/* Prototype des fonctions de lex/yacc (flex/bison) */

int yylex();
void yyerror(const char *);
int yyparse();
void prompt();

/* Message d'erreur en francais */

void Error(const char *);

%}

%union
{
    Expr *expr;
    int val;
    char *str;
}

%token <val> VAL
%token <str> IDENT
%token EOL

%type <expr> command affect expr

%left '+' '-'
%left '*' '/'
%left UNIOP

%%
lines   :       /* rien */
        |       lines command EOL
        |       error EOL
			{yyerrok; yyclearin; prompt();}
	;
command :       expr
			{cout << $1->eval() << endl; prompt();} 
        |       affect
			{$1->eval(); prompt();}
	| 
			{prompt();}
        ;
affect  :       IDENT '=' expr          
			{$$ = new Assignment(new Variable_Ref($1), $3);}
        ;
expr    :       VAL     
			{$$ = new Constant($1);}
        |       IDENT   
			{$$ = new Variable_Ref($1);}
        |       '(' expr ')'
                        {$$ = $2;}
	|       expr '?' expr ':' expr
			{$$ = new Ternary_Conditional($1, $3, $5);}
        |       expr '+' expr
			{$$ = new Binary_Plus($1, $3);}
        |       expr '-' expr
			{$$ = new Binary_Minus($1, $3);}
        |       expr '*' expr
			{$$ = new Binary_Mult($1, $3);}
        |       expr '/' expr
			{$$ = new Binary_Div($1, $3);}
        |       expr '%' expr
			{$$ = new Binary_Mod($1, $3);}
        |       '-' expr        %prec UNIOP
			{$$ = new Unary_Minus($2);}
        |       '+' expr        %prec UNIOP
			{$$ = new Unary_Plus($2);}
        ;
%%
	
#include "calculette_lex.cpp"

main()
{
    while (true) 
        try 
        {
            prompt();
            if (!yyparse()) exit(0);
        }
        catch (exception& e)
        {
            Error("valuation");
        }
}

void prompt()
{
    cout << "> " << flush;
}

void yyerror(const char *msg)
{
    // On ignore le message (en anglais) produit par yacc/bison
    
    Error("syntaxe");
}

void Error(const char *msg0)
{
    cerr << "Calculette: erreur: " << msg0 << endl;
}
