//=======================================================================
// Basic C++: a simple class template List
//      Specification
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: List.h,v 1.2 2004/11/01 15:19:04 jpr Exp $
//=======================================================================

#ifndef _LIST_H_
#define _LIST_H_

#include <iostream>
#include <cassert>
using namespace std;

#include "common_defs.h"

// Forward declarations(needed by g++-3.4)
template <typename ELEM>
class List;
template <typename ELEM>
ostream& operator<<(ostream&, const List<ELEM>&);

// Class List itself
template <typename ELEM>
class List
{
private:
    
   // The list Cell type 
    
    struct Cell
    {
        ELEM val;
        Cell *next;
        Cell(ELEM v, Cell *n = 0) : val(v), next(n) {}
        ~Cell() {delete next;} // Recursive destructor
    };

    // 
    Cell *head;         // Pointer to first cell
    Cell *tail;         // Pointer to last cell

public:
    // Exception

    class Empty {};

    // Construction

    List() : head(0), tail(0) {}

    ~List() {delete head;} // Destroy first cell (recursive destrucion)

    // Copy constructor and copy assignment
    List(const List&);
    List& operator=(const List&);

    // Accessor ?
    
    bool is_empty() const {return head == 0;}

    // Adding elements to the list
    
    void append(ELEM);  // at the end
    void prepend(ELEM);  // at the beginning

    // Removing elements from the list en tete de liste
    
    ELEM get_first();

    // Display (obey the g++ warning message and add <>!)
    
    friend ostream& operator<< <>(ostream&, const List&);
};

// Inclusion model for templates
#include "List.cpp"

#endif
