//=======================================================================
// Basic C++: Index et rfrences croises : la classe Index
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: Index.h,v 1.1 2005/05/03 12:50:31 jpr Exp $
//=======================================================================

#ifndef _INDEX_H_
#define _INDEX_H_

#include <iostream> 
#include <string>
#include <map>
#include <set>
using namespace std;

// La classe Index est juste une encapsulation de map<>

class Index
{
public:

    // Exception (mot non trouv)

    class Not_Found {};

    // Le type de la map<> interne

    typedef map<string, set<int> > Map;

    // Raccourci pour donner directement accs aux itrateurs de la map
    // On n'a pas besoin de parcourir pour modifier, donc on fournit juste un
    // const_iterator.

    typedef Map::const_iterator const_iterator;

    // Ajout d'un nom et d'un numro de ligne

    void add(const string& w, int ln) {_index[w].insert(ln);}

    // Accesseur pour obtenir l'ensemble des numros de ligne d'un mot

    const set<int>& operator[](const string& w) const;
 
    // Pour utiliser directement Index dans les algorithmes, comme si c'tait
    // une collection de la STL, on donne accs aux itrateurs et  la taille

    const_iterator begin() const {return _index.begin();}
    const_iterator end() const {return _index.end();}
    unsigned int size() const {return _index.size();}

    // Oprateur d'affichage : on ne peut pas utiliser la fonction display()
    // des collections, car set<> n'a pas d'oprateur <<

    friend ostream& operator<<(ostream& os, const Index& index);

private:

    // L'index lui-mme

    Map _index;
};

#endif
