/*=======================================================================
 Basic C++: classe Date
	A struct date in C (minimum)
-----------------------------------------------------------------------
 Jean-Paul Rigault --- Copyright 2003
 $Id: date_c.c,v 1.2 2004/11/07 10:52:52 jpr Exp $
=======================================================================*/

#include <time.h>
#include <stdio.h>

#include "date_c.h"

/*-----------------------------------------------------------------------
 Constants
-----------------------------------------------------------------------*/

/* POSIX localtime does not handle date earlier than 1900! */
#define MIN_YEAR 1900
/* Number of months */
#define NMONTHS 12
/* Maximum number of days in a month */
#define NDAYS 31

/* Number of days in months: Month[0] is for non leap year, 
   Month{1] for leap year */
static const int Days[][NMONTHS + 1] = {
    {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
};

static int is_leap(int y)
{
    if (y % 100 == 0) return y % 400 == 0;
    else return y % 4 == 0;
}

struct date today()
{
   time_t t = time(0);
   struct tm *ptm = localtime(&t);
   struct date dtemp;

   dtemp.year = ptm->tm_year + MIN_YEAR;
   dtemp.month = ptm->tm_mon + 1;
   dtemp.day = ptm->tm_mday;

   return dtemp;
}

struct date add(struct date dat, int n)
{
    int y;
    int m;
    int d;
    int i;
    struct date dtemp;

    for (i = 0, d = dat.day, m = dat.month, y = dat.year; i < n; ++i, ++d)
    {
	if (d >= Days[is_leap(y)][m]) /* we change month */
	{
	    d = 0; /* will be incremented before next iteration */
	    ++m;
	    if (m > NMONTHS) /* we change year also */
	    {
		++y;
		m = 1;
	    }
	}
    }
    
    dtemp.year = y;
    dtemp.month = m;
    dtemp.day = d;
    return dtemp;
}

struct date sub(struct date dat, int n)
{
    int y;
    int m;
    int d;
    int i;
    struct date dtemp;

    for (i = n, d = dat.day, m = dat.month, y = dat.year; i > 0; --i, --d)
    {
	if (d == 1) /* we change month */
	{
	    --m;
	    if (m <= 0) /* we change year also */
	    {
		--y;
		m = NMONTHS;
	    }
	    d = Days[is_leap(y)][m] + 1;
	    /* will be decremented before next iteration */
	}
    }

    dtemp.year = y;
    dtemp.month = m;
    dtemp.day = d;
    return dtemp;
}

void print_date(struct date dat)
{
    printf("%d/%d/%d\n", dat.month, dat.day, dat.year);
}
