//=======================================================================
// Basic C++: class Expression
//      Specification and implementation
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: Expr.h,v 1.3 2005/02/08 10:58:27 jpr Exp $
//=======================================================================

#ifndef _EXPR_H_
#define _EXPR_H_

#include "Expr-abstract.h"

// Constant expressions

class Constant : public Expr
{
private:
    const int val;
public:
    Constant(int v) : val(v) {}
    virtual int eval() const {return val;}
};

// Unary expressions

class Unary_Plus : public Unary_Expr
{
public:
    Unary_Plus(Expr *pe) : Unary_Expr(pe) {}
    virtual int eval() const {return op->eval();}
};

class Unary_Minus : public Unary_Expr
{
public:
    Unary_Minus(Expr *pe) : Unary_Expr(pe) {}
    virtual int eval() const {return -op->eval();}
};

// Binary expressions

class Binary_Plus : public Binary_Expr
{
public:
    Binary_Plus(Expr *pe1, Expr *pe2) : Binary_Expr(pe1, pe2) {}
    virtual int eval() const {return opl->eval() + opr->eval();}
};

class Binary_Minus : public Binary_Expr
{
public:
    Binary_Minus(Expr *pe1, Expr *pe2) : Binary_Expr(pe1, pe2) {}
    virtual int eval() const {return opl->eval() - opr->eval();}
};

class Binary_Mult : public Binary_Expr
{
public:
    Binary_Mult(Expr *pe1, Expr *pe2) : Binary_Expr(pe1, pe2) {}
    virtual int eval() const {return opl->eval() * opr->eval();}
};

class Binary_Div : public Binary_Expr
{
public:
    Binary_Div(Expr *pe1, Expr *pe2) : Binary_Expr(pe1, pe2) {}
    virtual int eval() const
    {
        int d = opr->eval();
        if (d == 0) throw ZeroDivide();
        return opl->eval() / d;
    }
};

#endif
