//=======================================================================
// Basic C++: a simple class template List
//      Implementation
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: List.cpp,v 1.2 2004/11/01 15:19:04 jpr Exp $
//=======================================================================

#include "List.h"

//-----------------------------------------------------------------------
// Copy constructor and copy assignment
//-----------------------------------------------------------------------
    
template <typename ELEM>
List<ELEM>::List(const List& l)
    : head(0), tail(0)
{
    for (Cell *pcell = l.head; pcell != 0; pcell = pcell->next)
        append(pcell->val);
}

template <typename ELEM>
List<ELEM>& List<ELEM>::operator=(const List& l)
{
    if (this != &l)
    {
        delete head;
        head = tail = 0;
        for (Cell *pcell = l.head; pcell != 0; pcell = pcell->next)
            append(pcell->val);
    }
    return *this;       
}

//-----------------------------------------------------------------------
// Adding elements to the list
//-----------------------------------------------------------------------

template <typename ELEM>
void List<ELEM>::append(ELEM v)
{
    Cell *p = new Cell(v);
    if (head == 0) head = tail = p;
    else
    {
        tail->next = p;
        tail = p;
    }
}

template <typename ELEM>
void List<ELEM>::prepend(ELEM v)
{
    Cell *p = new Cell(v);
    if (head == 0) head = tail = p;
    else
    {
        p->next = head;
        head = p;
    }
}

// Find the first elemeent in the list greater than e and insert before

template <typename ELEM>
void List<ELEM>::insert(ELEM e)
{
    Cell *p, *pprev;

    for (p = head, pprev = 0;
         p != 0 && p->val < e; pprev = p, p = p->next) {}

    if (pprev != 0) // inserting in the middle or at the end
    {
        Cell *ptemp = new Cell(e, pprev->next);
        pprev->next = ptemp;
        if (p == 0) tail = ptemp;
    }
    else prepend(e);   // inserting in front
}

//-----------------------------------------------------------------------
// Removing elements from the list
//-----------------------------------------------------------------------

template <typename ELEM>
ELEM List<ELEM>::get_first()
{
    if (head == 0) throw Empty();
    ELEM v = head->val;
    if (tail == head) tail = 0;
    Cell *p = head;
    head = head->next;
    p->next = 0; // needed: otherwise the next line destroy the whole list!
    delete p;
    return v;
}

//-----------------------------------------------------------------------
// Display
//-----------------------------------------------------------------------

template <typename ELEM>
ostream& operator<<(ostream& os, const List<ELEM>& l)
{
    typename List<ELEM>::Cell *p;
    for (p = l.head; p != 0; p = p->next) cout << p->val << ' ';
    return os;
}
