//=======================================================================
// Basic C++: Using STL iterators and algorithms for analysing word
//  occurrences in a text// 
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2005
// $Id: word_freq.cpp,v 1.1 2005/02/24 18:13:56 jpr Exp $
//=======================================================================

#include <iostream>
#include <string>
#include <list>
#include <vector>
#include <map>
#include <algorithm>
using namespace std;

// Forward declarations

template <typename K, typename T>
void print_pair(pair<K, T> p);

template <typename T>
void print_simple(T t);

bool several_occ(pair<string, int> p);

template <typename K, typename T>
K proj1(pair<K, T> p);
template <typename K, typename T>
T proj2(pair<K, T> p);

template <int N>
bool wshorter(string s);

// Main program

int main()
{
    // Read the standard input word by word and stor in map, 
    // incrementing occurrence count
    map<string, int> the_map;
    string s;
    while (cin >> s) the_map[s]++;

    // Print the result
    cout << "Occurrences per word\n";
    cout << "--------------------\n";
    for_each(the_map.begin(), the_map.end(), print_pair<string, int>);

    // Print all words with several occurrences
    cout << endl;
    cout << "Words with several occurrences\n";
    cout << "------------------------------\n";  
    list<pair<string, int> > loccs;
    for (map<string, int>::iterator it = the_map.begin();
	 (it = find_if(it, the_map.end(), several_occ)) != the_map.end();
	  ++it)
    {
	print_pair(*it);
	loccs.push_back(*it);
    }

    // Compute the average lenght
    cout << endl;
    cout << "Average length : ";
    int num = 0;
    int denom = 0;
    for (map<string, int>::iterator it = the_map.begin();
	 it != the_map.end(); ++it)
    {
	num += it->second * it->first.size();
	denom += it->second;
    }
    cout << double(num) / double(denom) << '\n';
    cout << "----------------\n"; 

    // Transform the list of map cells (loccs above) into a list of words
    vector<string> vec(loccs.size());
    transform(loccs.begin(), loccs.end(), vec.begin(), proj1<string, int>);
    cout << "Words with several occurrences (as a list)\n";
    cout << "------------------------------------------\n";  
    for_each(vec.begin(), vec.end(), print_simple<string>);

    // Eliminate with no more that 3 characters
    vec.erase(remove_if(vec.begin(), vec.end(), wshorter<3>), vec.end());
    cout << "Words with several occurrences longer than 3 characters\n";
    cout << "-------------------------------------------------------\n";  
    for_each(vec.begin(), vec.end(), print_simple<string>);   
}

//-----------------------------------------------------------------------
// Auxiliary functions for use with the algorithms
//-----------------------------------------------------------------------

// Print a map element
template <typename K, typename T>
void print_pair(pair<K, T> p)
{
    cout << p.first << " : " << p.second << '\n';
}

// Print an element of a simple type
template <typename T>
void print_simple(T t)
{
    cout << t << '\n';
}

// Projection of pairs
template <typename K, typename T>
K proj1(pair<K, T> p)
{
   return p.first;
}
template <typename K, typename T>
T proj2(pair<K, T> p)
{
   return p.second;
}

// Look for a word with more than 1 occurrence
bool several_occ(pair<string, int> p)
{
    return p.second > 1;
}

// Decide whether a string is short
template <int N>
bool wshorter(string s)
{
    return s.size() <= N;
}
