//=======================================================================
// Basic C++: collections polymorphes (figures gomtriques)
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id$
//=======================================================================

#ifndef _FIGURE_H_
#define _FIGURE_H_

#include <iostream>
using namespace std;


const double PI = 3.141592;

//-----------------------------------------------------------------------
//: Classe Point (pour le centre des figures)
//-----------------------------------------------------------------------

struct Point
{
    double _x;
    double _y;

    Point(double x, double y) : _x(x), _y(y)
    {}

    friend ostream& operator<<(ostream& os, const Point& pt)
    {
	return os << "(" << pt._x << ", " << pt._y << ")";
    }
};

//-----------------------------------------------------------------------
//: Classe Figure (abstraite)
//-----------------------------------------------------------------------

class Figure
{
private:

    Point _center;
    double _angle;	// angle avec l'horizontale

public:

    Figure(Point c, double a = 0.0) : _center(c), _angle(0)
    {}

    virtual ~Figure() {}

    void move(double dx, double dy)
    {
	_center._x += dx;
	_center._y += dy;
    }

    virtual void rotate(double alpha)
    {
	_angle += alpha;
    }
	
    virtual void scale(double s) = 0;

    virtual double surface() const = 0;

    friend ostream& operator<<(ostream& os, const Figure& fig)
    {
	os << "Centre: " << fig._center << " angle: " << fig._angle << " ";
	fig._display(os);
	return os;
    }

private:

    virtual void _display(ostream& os) const = 0;

};

//-----------------------------------------------------------------------
//: Figures Concrtes
//-----------------------------------------------------------------------

class Ellipse : public Figure
{
private:

    double _a;
    double _b;

public:
    Ellipse(Point c, double a, double b) : Figure(c), _a(a), _b(b)
    {}

    double a_axis() const {return _a;}

    double b_axis() const {return _b;}

    virtual void scale(double s)
    {
	_a *= s;
	_b *= s;
    }

    virtual double surface() const 
    {
	return PI * _a * _b;
    }

private:

    virtual void _display(ostream& os) const 
    {
	os << "Ellipse: axes (" << _a << ", " << _b << ")";
    }
};

class Circle : public Ellipse
{
public:

    Circle(Point c, double r) : Ellipse(c, r, r)
    {}

    double radius() const {return a_axis();}

    virtual void rotate(double alpha)
    {
	// Rien  faire
    }

private:

    virtual void _display(ostream& os) const 
    {
	os << "Cercle: rayon " << radius();
    }
};

class Rectangle : public Figure
{
private:

    double _w;
    double _h;

public:

    Rectangle(Point c, double w, double h) : Figure(c), _w(w), _h(h)
    {}

    double width() const {return _w;}

    double heigth() const {return _h;}

    virtual void scale(double s) 
    {
	_w *= s;
	_h *= s;
    }

    virtual double surface() const 
    {
	return _w * _h;
    }

private:

    virtual void _display(ostream& os) const 
    {
	os << "Rectangle: cts (" << _w << ", " << _h << ")";;
    }
};

class Square : public Rectangle
{
public:

    Square(Point c, double a) : Rectangle(c, a, a)
    {}

    double side() const {return width();}

private:

    virtual void _display(ostream& os) const 
    {
	os << "Carr: ct " << side();
    }
};

#endif
